/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BuilderService.java 21613 2011-08-10 13:46:31Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.admin.autostart.service;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Get;
import org.ow2.jonas.admin.autostart.vo.StarterVO;
import org.ow2.jonas.admin.autostart.vo.configuration.JDBCConfigurationVO;
import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.builder.BuilderException;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.configuration.JDBCDatasource;
import org.ow2.jonas.autostart.configuration.Jms;
import org.ow2.jonas.autostart.configuration.Protocol;
import org.ow2.jonas.autostart.configuration.Service;
import org.ow2.jonas.autostart.parser.Parser;
import org.ow2.jonas.autostart.parseurl.ParseUrl;
import org.ow2.jonas.autostart.proxy.AutoStartProxy;
import org.ow2.jonas.autostart.utility.Output;
import org.ow2.jonas.autostart.utility.Utility;
import org.ow2.jonas.lib.management.extensions.manager.ManagementEntryPoint;
import org.ow2.jonas.tools.configurator.api.JDBCConfiguration;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 *
 */
public class BuilderService implements Serializable {

    private transient Thread buildThread = null;

    // private Builder builder = null;
    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 3098492588250932378L;

    /**
     * The builder librairy object.
     */
    // private BuilderVO builderVo = new BuilderVO();
    private BuilderInfo builderVo = null;

    /**
     * The build process status.
     */
    private String buildStatus = "Error";

    /**
     * The EZB JMX server.
     */
    private transient MBeanServerConnection serv = null;

    /**
     * Domain name.
     */
    private String domainName = null;

    /**
     * Server name.
     */
    private String serverName = null;

    /**
     * JOnAS Base.
     */
    private String jonasBase = null;

    /**
     * JOnAS Root.
     */
    private String jonasRoot = null;

    /**
     * The file separator.
     */
    private final String FILE_SEPARATOR = System.getProperty("file.separator");

    /**
     * The upload folder.
     */
    private final String UPLOAD_FOLDER = "builderUploadDir";

    /**
     * The folder where the starter should be copied.
     */
    private transient File STARTERS_DEST_LOCATION = null;

    /**
     * The jonas-starter jar file name.
     */
    private final String STARTER_JAR_FILE = "jonas-starter.jar";

    /**
     * The xml file where starter details should be saved.
     */
    private transient File starterDetailFile = null;

    private File STARTER_DETAIL_FILE = null;

    /**
     * The starters jar file folder.
     */
    private String startersFolder = null;

    /**
     * The zip file option.
     */
    final int ZIP_FILE_OPTION = 1;

    /**
     * The current JOnAS folder(s) option.
     */
    final int CURRENT_JONAS_FOLDER = 2;

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(BuilderService.class);

    public BuilderService() {
    }

    private void init() {
        Util.init();
        startersFolder = Util.getJonasBase() == null ? Util.getJonasRoot() : Util.getJonasBase();
        startersFolder = startersFolder + FILE_SEPARATOR + "starters";
        if (!new File(startersFolder).exists()) {
            Utility.createDirectory(new File(startersFolder));
        }
        starterDetailFile = new File(startersFolder, "starter-infos.xml");
        if (!starterDetailFile.exists()) {
            Utility.createNewFile(starterDetailFile);
            Utility.editFile(starterDetailFile, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<starters/>");
        }
        STARTERS_DEST_LOCATION = new File(startersFolder);
        STARTER_DETAIL_FILE = starterDetailFile;
    }

    /**
     * Launch the build process.
     *
     * @param builderVo
     * @return the build status
     */
    public String startBuildProcess(final BuilderInfo builderInfo) {

        this.builderVo = builderInfo;
        this.builderVo.starters = builderInfo.starters;

        // this.builder = new Builder();
        buildThread = new Thread() {
            @Override
            public void run() {

                String uploadDirectory = Utility.getDefaultLocation() + FILE_SEPARATOR + UPLOAD_FOLDER;

                String url = null;
                Output output = new Output(builderVo.outputText);
                Builder builder = new Builder();
                String applications = "";

                switch (builderVo.getJonasFrom) {

                    case ZIP_FILE_OPTION:
                        url = uploadDirectory + FILE_SEPARATOR + builderVo.jonasZipFile;
                        if (!new File(url).exists()) {
                            url = null;
                            buildStatus = builderVo.jonasZipFile + "##";
                        }
                        break;

                    case CURRENT_JONAS_FOLDER:
                        init();
                        builder.setJonasRoot(new File(jonasRoot));
                        if (!jonasRoot.equals(jonasBase)) {
                            builder.setJonasBase(new File(jonasBase));
                        }
                        url = jonasRoot;
                        break;

                    default:
                        String type = builderVo.jonasType;
                        String version = builderVo.jonasVersion;
                        url = createJOnASUrl(type, version);
                        try {
                            if (!Utility.testConnexionURL(new URL(url))) {
                                url = null;
                            }
                        } catch (MalformedURLException e) {
                            logger.error("Error while using the url '" + url + "'", e);
                            buildStatus = e.getMessage();
                        }
                }

                if (url != null) {

                    // Set the application absolute path.
                    String jonasBaseDeployFolder = jonasBase + FILE_SEPARATOR + "deploy";
                    String jonasRootDeployFolder = jonasRoot + FILE_SEPARATOR + "deploy";
                    for (String application : builderVo.applications) {
                        if (new File(uploadDirectory, application).exists()) {
                            applications += uploadDirectory + FILE_SEPARATOR + application + ";";
                        } else if (new File(jonasBaseDeployFolder, application).exists()) {
                            applications += jonasBaseDeployFolder + FILE_SEPARATOR + application + ";";
                        } else if (new File(jonasRootDeployFolder, application).exists()) {
                            applications += jonasRootDeployFolder + FILE_SEPARATOR + application + ";";
                        }
                    }

                    builder.setVerboseMode(true);
                    builder.createTmpWorkDirectory();
                    builder.setApplicationLocation(applications);
                    builder.setUrl(url);
                    builder.setIncludeJonasClient(builderVo.includeJonasClient);
                    builder.setUseJonasFullStarter(builderVo.useFullStarter);
                    builder.setOutput(output);
                    if (STARTERS_DEST_LOCATION != null) {
                        if (!STARTERS_DEST_LOCATION.exists()) {
                            init();
                            Utility.createDirectory(STARTERS_DEST_LOCATION);
                        }
                    } else {
                        init();
                        Utility.createDirectory(STARTERS_DEST_LOCATION);
                    }

                    builder.setStarterJarFileLocation(STARTERS_DEST_LOCATION);
                    if (builderVo.jonasStarterName.equals("")) {
                        builderVo.jonasStarterName = STARTER_JAR_FILE;
                    }
                    builder.setStarterJarFileName(builderVo.jonasStarterName);

                    if (builderVo.applyConfiguration) {
                        Configuration configuration = getConfigurationValue();
                        builder.setConfiguration(configuration);
                        builder.setCreateNewJonasBase(true);
                    }
                    builder.setVerboseMode(false);
                    try {
                        builder.createJonas();
                    } catch (BuilderException e) {
                        logger.error("Error while creating JOnAS", e);
                        throw new IllegalStateException("Cannot create JOnAS instance", e);
                    }
                    try {
                        builder.setStarter();
                    } catch (BuilderException e) {
                        logger.error("Error while getting JOnAS starter", e);
                        throw new IllegalStateException("Cannot get JOnAS starter", e);
                    }
                    try {
                        builder.unzipStarter();
                    } catch (BuilderException e) {
                        logger.error("Error while unzip JOnAS starter", e);
                        throw new IllegalStateException("Cannot unzip JOnAS starter", e);
                    }
                    builder.injectJonasIntoStarter();
                    builder.zipStarter();
                    buildStatus = "Success";
                    Utility.deleteDirectory(builder.getWorkdirectory());
                } else {
                    // buildStatus = "Error22";
                }
            }
        };
        buildThread.start();
        while (buildThread.isAlive()) {
            Thread.yield();
        }

        return buildStatus;
    }

    public void stopBuildProcess() {
        if (buildThread != null) {
            buildThread.interrupt();
        }
    }

    /**
     * @return the configuration value.
     */
    private Configuration getConfigurationValue() {
        String uploadDirectory = Utility.getDefaultLocation() + FILE_SEPARATOR + UPLOAD_FOLDER;
        Configuration configuration = new Configuration();
        /**
         * HSQL DB
         */
        configuration.setDbName(builderVo.configuration.hsqlDbVo.dbName);
        configuration.setDbPort(builderVo.configuration.hsqlDbVo.dbPort);
        configuration.setDbUser(builderVo.configuration.hsqlDbVo.dbUser);

        /**
         * Web container.
         */
        configuration.setHttpPort(builderVo.configuration.webContainerVo.httpPort);
        configuration.setRedirectPort(builderVo.configuration.webContainerVo.redirectPort);
        configuration.setJvmRouteActivation(builderVo.configuration.webContainerVo.activateJvmRoute);
        configuration.setHttpsConnectorActivation(builderVo.configuration.webContainerVo.useSsl);

        /**
         * Carol protocols.
         */
        Vector<Protocol> protocol = new Vector<Protocol>();
        for (String prot : builderVo.configuration.protocolsVo.protocolList) {
            Protocol p = new Protocol();
            p.setName(prot);
            p.setIsEnabled(true);
            if (prot.equalsIgnoreCase("jrmp")) {
                p.setPort(builderVo.configuration.protocolsVo.jrmpPort);
            } else if (prot.equalsIgnoreCase("irmi")) {
                p.setPort(builderVo.configuration.protocolsVo.irmiPort);
            } else if (prot.equalsIgnoreCase("lmi")) {
                p.setPort(builderVo.configuration.protocolsVo.lmiPort);
            } else if (prot.equalsIgnoreCase("iiop")) {
                p.setPort(builderVo.configuration.protocolsVo.iiopPort);
            }
            protocol.add(p);
        }
        configuration.setProtocols(protocol);

        /**
         * Services.
         */
        Vector<Service> services = new Vector<Service>();
        for (String srv : builderVo.configuration.servicesVo.servicesList) {
            Service service = new Service();
            service.setName(srv);
            service.setIsEnabled(true);
            services.add(service);
        }
        configuration.setServices(services);

        /**
         * Jms.
         */
        Jms jms = new Jms();
        jms.setPort(builderVo.configuration.jmsVo.jmsPort);

        Vector<String> queues = new Vector<String>();
        for (String queue : builderVo.configuration.jmsVo.queueList) {
            queues.add(queue);
        }
        jms.setQueues(queues);

        Vector<String> topics = new Vector<String>();
        for (String topic : builderVo.configuration.jmsVo.topicList) {
            topics.add(topic);
        }
        jms.setTopics(topics);
        configuration.setJms(jms);

        for (JDBCConfigurationVO jdbcDs : builderVo.configuration.jdbcConfigurationVO) {
            JDBCDatasource datasource = new JDBCDatasource();
            String rarName = jdbcDs.rarName.equals("") ? jdbcDs.datasourceName : jdbcDs.rarName;
            datasource.setRaName(rarName);
            datasource.setDatasourceName(jdbcDs.datasourceName);
            datasource.setDbmsName(jdbcDs.dbmsName);

            // Others attributs
            JDBCConfiguration dsJdbcConfig = new JDBCConfiguration();
            dsJdbcConfig.datasourceClass = jdbcDs.dsClassName;
            dsJdbcConfig.jndiName = jdbcDs.jndiName;
            dsJdbcConfig.driverName = uploadDirectory + FILE_SEPARATOR + jdbcDs.driverName;
            dsJdbcConfig.mappername = jdbcDs.dsMapper;
            dsJdbcConfig.url = jdbcDs.dsUrl;
            dsJdbcConfig.user = jdbcDs.dsUser;
            dsJdbcConfig.password = jdbcDs.dsUserPasswd;
            dsJdbcConfig.rarLink = jdbcDs.rarLink;

            // Advanced attributs
            dsJdbcConfig.checkLevel = jdbcDs.advancedJDBCConfiguration.checkingLevel;
            dsJdbcConfig.poolInitialSize = jdbcDs.advancedJDBCConfiguration.adjustPeriod;
            dsJdbcConfig.poolMaximumAgeMinutes = jdbcDs.advancedJDBCConfiguration.maxAge;
            dsJdbcConfig.poolMaximumOpenTime = jdbcDs.advancedJDBCConfiguration.maxOpenTime;
            dsJdbcConfig.poolMaximumSize = jdbcDs.advancedJDBCConfiguration.maxPool;
            dsJdbcConfig.poolMaximumWaitTime = jdbcDs.advancedJDBCConfiguration.maxWaitTime;
            dsJdbcConfig.poolMaximumNumberOfWaiters = jdbcDs.advancedJDBCConfiguration.maxWaiters;
            dsJdbcConfig.poolMinimumSize = jdbcDs.advancedJDBCConfiguration.minPool;
            dsJdbcConfig.poolMaximumNumberOfPreparedStatements = jdbcDs.advancedJDBCConfiguration.pstmtPoolMin;
            dsJdbcConfig.poolSamplingPeriod = jdbcDs.advancedJDBCConfiguration.samplingPeriod;
            dsJdbcConfig.testStatement = jdbcDs.advancedJDBCConfiguration.test;

            datasource.setJdbcConfiguration(dsJdbcConfig);
            configuration.addJdbcDatasource(datasource);
        }
        return configuration;
    }

    /**
     * Return the output text content to fill Flex module textarea.
     *
     * @return ouputTextContent.
     */
    public String getOuputTextContent() {
        ArrayList<String> outputContent = builderVo.outputText;
        int index = outputContent.size();
        if (index > 0) {
            return outputContent.get(index - 1);
        }
        return "";
    }

    /**
     * @return the latest version of JOnAS.
     */
    public List<String> getJonasVersions() {

        try {
            if (Utility.testConnexionURL(new URL("http://repo1.maven.org/maven2/org/ow2/"
                + "jonas/assemblies/maven-metadata.xml"))) {
                Utility.setProxyValues(new AutoStartProxy());
                Parser parser = new Parser("http://repo1.maven.org/maven2/org/ow2/" + "jonas/assemblies/maven-metadata.xml",
                    "", "JOnAS");
                return parser.getAllLatestVersions();
            }
        } catch (MalformedURLException ex) {
            logger.error("Error while getting JOnAS versions", ex);
            Logger.getLogger(BuilderService.class.getName()).log(Level.SEVERE, null, ex);
        }
        return new ArrayList<String>();
    }

    /**
     * @return the deployed applications in $JONAS_BASE/deploy.
     */
    public ArrayList<String> getAvailableApplication() throws IOException {
        ArrayList<String> availableApplications = new ArrayList<String>();
        domainName = ManagementEntryPoint.getInstance().getDomainName();
        serverName = ManagementEntryPoint.getInstance().getServerName();

        serv = ManagementEntryPoint.getInstance().getServerConnection();

        ObjectName j2eeServer = null;
        try {
            j2eeServer = new ObjectName(domainName + ":j2eeType=J2EEServer,name=" + serverName);
        } catch (MalformedObjectNameException e) {
            logger.error("Error while getting all applications", e);
        }

        try {
            jonasBase = (String) serv.getAttribute(j2eeServer, "jonasBase");
            jonasRoot = (String) serv.getAttribute(j2eeServer, "jonasRoot");
            // Get WAR applications
            String deployedWars = serv.getAttribute(j2eeServer, "deployedWars").toString();
            deployedWars = deployedWars.substring(1, deployedWars.length() - 1);
            String[] warApplications = deployedWars.split(",");
            for (String application : warApplications) {
                if (!application.equals("")) {
                    availableApplications.add(application.substring(application.lastIndexOf(FILE_SEPARATOR) + 1, application
                        .length()));
                }
            }

            // Get EAR applications
            String deployedEars = serv.getAttribute(j2eeServer, "deployedEars").toString();
            deployedEars = deployedEars.substring(1, deployedEars.length() - 1);
            String[] earApplications = deployedEars.split(",");
            for (String application : earApplications) {
                if (!application.equals("")) {
                    availableApplications.add(application.substring(application.lastIndexOf(FILE_SEPARATOR) + 1, application
                        .length()));
                }
            }

        } catch (InstanceNotFoundException e) {
            logger.error("Error while getting all applications", e);
        } catch (ReflectionException e) {
            logger.error("Error while getting all applications", e);
        } catch (IOException e) {
            logger.error("Error while getting all applications", e);
        } catch (MBeanException e) {
            logger.error("Error while getting all applications", e);
        } catch (AttributeNotFoundException e) {
            logger.error("Error while getting all applications", e);
        }
        return availableApplications;
    }

    /**
     * Download a file.
     *
     * @param url the file URL
     * @return the download status.
     */
    public String downloadFile(final String url) {
        Get get = new Get();
        Project project = new Project();
        try {
            get.setSrc(new URL(url));
        } catch (MalformedURLException ex) {
            logger.error("Error while downloading file '" + url + "'", ex);
        }
        String tmpDir = Utility.getDefaultLocation();
        Utility.createDirectory(new File(tmpDir, UPLOAD_FOLDER));
        String workdirectory = tmpDir + System.getProperty("file.separator") + UPLOAD_FOLDER;
        String file = url.substring(url.lastIndexOf("/") + 1);
        get.setDest(new File(workdirectory, file));
        get.setProject(project);
        get.execute();
        if (new File(workdirectory, file).exists()) {
            return file;
        }
        return "null";
    }

    /**
     * Check if the URL is valid.
     *
     * @param url the URL to chjeck.
     * @return true if the URL is valid, false otherwise.
     */
    public boolean isValidUrl(final String url) {
        try {
            if (Utility.testConnexionURL(new URL(url))) {
                return true;
            }
        } catch (MalformedURLException ex) {
            logger.error("Error while testing url '" + url + "'", ex);
            builderVo.outputText.add(ex.getMessage());
        }
        return false;
    }

    /**
     * Create a URL for gettion JOnAS server based on choosen version and type.
     *
     * @param type the type of JOnAS server to download.
     * @param version the version of JOnAS server to download.
     * @return JOnAS url.
     */
    private String createJOnASUrl(final String type, final String version) {
        Utility.setProxyValues(new AutoStartProxy());
        Parser parser = new Parser("http://repo1.maven.org/maven2/org/ow2/" + "jonas/assemblies/maven-metadata.xml", "",
            "JOnAS");
        ParseUrl jonasZipFile = new ParseUrl(parser, "http://repo1.maven.org/maven2/org/ow2/jonas/" + "assemblies/profiles/",
            "-bin.zip", type);
        return jonasZipFile.getUrl(version);
    }

    /**
     * Save starter details in an xml file.
     *
     * @return the file location.
     * @param builderInfo the builder object.
     */
    public String saveStarter(final BuilderInfo builderInfo) {
        this.builderVo = builderInfo;
        DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();

        DocumentBuilder constructeur = null;
        try {
            constructeur = fabrique.newDocumentBuilder();
        } catch (ParserConfigurationException ex) {
            logger.error("Error while creating builder", ex);
            return null;
        }
        Document document = constructeur.newDocument();
        Element root = document.createElement("starters");
        for (StarterVO starterVo : this.builderVo.starters) {
            Element starter = document.createElement("starter");
            root.appendChild(starter);
            Element name = document.createElement("name");
            if (!starterVo.jonasStarterName.endsWith(".jar")) {
                starterVo.jonasStarterName += ".jar";
            }
            name.setTextContent(starterVo.jonasStarterName);
            starter.appendChild(name);
            Element path = document.createElement("path");
            starterVo.jonasStarterPath = STARTERS_DEST_LOCATION.getAbsolutePath() + FILE_SEPARATOR + starterVo.jonasStarterName;
            path.setTextContent(starterVo.jonasStarterPath);
            starter.appendChild(path);
            Element date = document.createElement("date");
            date.setTextContent(starterVo.jonasStarterCreationDate);
            starter.appendChild(date);
            Element description = document.createElement("description");
            description.setTextContent(starterVo.jonasStarterDescription);
            starter.appendChild(description);
            Element applications = document.createElement("applications");
            starter.appendChild(applications);
            for (String applicationVo : starterVo.applications) {
                Element application = document.createElement("application");
                application.setTextContent(applicationVo);
                applications.appendChild(application);
            }
        }
        document.appendChild(root);
        try {
            Source source = new DOMSource(document);
            Utility.createNewFile(STARTER_DETAIL_FILE);
            Result result = new StreamResult(STARTER_DETAIL_FILE);
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
            transformer.transform(source, result);
        } catch (Exception e) {
            logger.error("Error while transforming XML", e);
        }
        return "";
    }

    /**
     * Load starter details from a file.
     *
     * @return starterVo.
     */
    public List<StarterVO> loadStarters() {
        List<StarterVO> loadedStarters = new ArrayList<StarterVO>();
        if (STARTER_DETAIL_FILE == null) {
            init();
        }
        if (STARTER_DETAIL_FILE.exists()) {
            DocumentBuilder docBuilder = null;
            DocumentBuilderFactory builyderFactory = DocumentBuilderFactory.newInstance();
            try {
                docBuilder = builyderFactory.newDocumentBuilder();
            } catch (ParserConfigurationException ex) {
                logger.error("Error while creating builder", ex);
            }
            Document document = null;
            try {
                document = docBuilder.parse(STARTER_DETAIL_FILE);
                NodeList nodes = document.getDocumentElement().getChildNodes();
                for (int i = 0; i < nodes.getLength(); i++) {
                    Node starters = nodes.item(i);
                    if (starters.getNodeType() == Node.ELEMENT_NODE && ((Element) starters).getTagName().equals("starter")) {
                        StarterVO starterVo = new StarterVO();
                        List<String> jonasStarterApplications = new ArrayList<String>();
                        Element starter = (Element) starters;
                        NodeList nodes_j = starter.getChildNodes();
                        for (int j = 0; j < nodes_j.getLength(); j++) {
                            Node node_j = nodes_j.item(j);
                            if (node_j.getNodeType() == Node.ELEMENT_NODE) {
                                Element pieces = (Element) node_j;

                                if (pieces.getTagName().equals("name")) {
                                    starterVo.jonasStarterName = pieces.getTextContent();
                                } else if (pieces.getTagName().equals("path")) {
                                    starterVo.jonasStarterPath = pieces.getTextContent();
                                } else if (pieces.getTagName().equals("date")) {
                                    starterVo.jonasStarterCreationDate = pieces.getTextContent();
                                } else if (pieces.getTagName().equals("description")) {
                                    starterVo.jonasStarterDescription = pieces.getTextContent();
                                } else if (pieces.getTagName().equals("applications")) {
                                    Element application = pieces;
                                    NodeList nodes_k = application.getChildNodes();
                                    for (int k = 0; k < nodes_k.getLength(); k++) {
                                        Node appli = nodes_k.item(k);
                                        if (appli.getNodeType() == Node.ELEMENT_NODE) {
                                            jonasStarterApplications.add(appli.getTextContent());
                                        }
                                    }
                                    starterVo.applications = jonasStarterApplications;
                                }
                            }
                        }
                        loadedStarters.add(starterVo);
                    }
                }
            } catch (SAXException ex) {
                logger.error("Error while creating builder", ex);
            } catch (IOException ex) {
                logger.error("Error while creating builder", ex);
            }
        }
        return loadedStarters;
    }

    /**
     * Return the services list.
     *
     * @return services lis.
     */
    public ArrayList<String> getJonasServices() {

        Service jtm = new Service("jtm", false);
        Service db = new Service("db", false);
        Service dbm = new Service("dbm", false);
        Service security = new Service("security", false);
        Service resource = new Service("resource", false);
        Service maildb = new Service("mail", false);
        Service cmi = new Service("cmi", false);
        Service ha = new Service("ha", false);

        Service versioning = new Service("versioning", false);
        Service ejb2 = new Service("ejb2", false);
        Service ejb3 = new Service("ejb3", false);
        Service jaxrpc = new Service("jaxrpc", false);
        Service jaxws = new Service("jaxws", false);
        Service web = new Service("web", false);
        Service ear = new Service("ear", false);
        Service depmonitor = new Service("depmonitor", true);

        Service discovery = new Service("discovery", false);
        Service resourcemonitor = new Service("resourcemonitor", false);
        Service smartclient = new Service("smartclient", false);
        Service audit = new Service("audit", false);
        Service cdi = new Service("cdi", false);

        ArrayList<String> services = new ArrayList<String>();
        services.add(jtm.getName());
        services.add(db.getName());
        services.add(dbm.getName());
        services.add(security.getName());
        services.add(resource.getName());
        services.add(maildb.getName());
        services.add(cmi.getName());
        services.add(ha.getName());
        services.add(versioning.getName());
        services.add(ejb2.getName());
        services.add(ejb3.getName());
        services.add(jaxrpc.getName());
        services.add(jaxws.getName());
        services.add(web.getName());
        services.add(ear.getName());
        services.add(depmonitor.getName());
        services.add(discovery.getName());
        services.add(resourcemonitor.getName());
        services.add(smartclient.getName());
        services.add(audit.getName());
        services.add(cdi.getName());
        Collections.sort(services);
        return services;
    }

    /**
     * Set the starters path.
     *
     * @param builderInfo the builder object.
     * @return starterVo list.
     */
    public List<StarterVO> setStartersPath(final BuilderInfo builderInfo) {
        this.builderVo = builderInfo;
        List<StarterVO> jonasStarters = new ArrayList<StarterVO>();
        for (StarterVO starter : this.builderVo.starters) {
            starter.jonasStarterPath = STARTERS_DEST_LOCATION + FILE_SEPARATOR + starter.jonasStarterName;
            jonasStarters.add(starter);
        }
        return jonasStarters;
    }

    /**
     * Delete a starter from disc.
     *
     * @param starterName the the starter to delete.
     * @return the delete status.
     */
    public String deleteStarter(final String starterName) {
        init();
        File[] files = STARTERS_DEST_LOCATION.listFiles();
        for (int i = 0; i < files.length; i++) {
            if (files[i].getName().equals(starterName)) {
                files[i].setExecutable(true, false);
                files[i].setWritable(true, false);
                files[i].delete();
                return "true";
            }
        }
        return "false";
    }

    /**
     * @return the current JOnAS Base and Root folders.
     */
    public List<String> getJonasFolders() {
        List<String> folders = new ArrayList<String>();
        Util.init();
        folders.add(Util.getJonasRoot());
        folders.add(Util.getJonasBase());
        return folders;
    }
}
